@@ -9,7 +9,6 @@ with open(os.path.join(__path__[0], '__init__.py')) as f:
 import sys
 sys.setrecursionlimit(10000)
 import ast
-import copy
 import json
 import logging
 import warnings
@@ -35,7 +34,7 @@ tf.get_logger().setLevel(logging.FATAL)
 
 import onnx
 import onnx_graphsurgeon as gs
-from typing import Optional, List, Any
+from typing import Optional, List, Any, Dict
 from argparse import ArgumentParser
 
 import importlib
@@ -45,6 +44,8 @@ from onnx2tf.utils.common_functions import (
     onnx_tf_tensor_validation,
     weights_export,
     download_test_image_data,
+    get_tf_model_inputs,
+    get_tf_model_outputs,
 )
 from onnx2tf.utils.colors import Color
 from sng4onnx import generate as op_name_auto_generate
@@ -599,7 +600,7 @@ def convert(
             output_op_name \
                 for output_op_name in output_names_to_interrupt_model_conversion
         ]
-        onnx_graph = extraction(
+        onnx_graph: onnx.ModelProto = extraction(
             input_op_names=[graph_input.name for graph_input in graph.inputs],
             output_op_names=output_names,
             onnx_graph=onnx_graph,
@@ -608,6 +609,34 @@ def convert(
         del graph
         graph = gs.import_onnx(onnx_graph)
 
+    # ONNX dummy inference
+    # Generate output for all OPs.
+    # Used to verify the output error of each OP in the TensorFlow model.
+    full_ops_output_names = []
+    for graph_node in graph.nodes:
+        full_ops_output_names_sub = []
+        for graph_node_output in graph_node.outputs:
+            full_ops_output_names_sub.append(graph_node_output.name)
+        full_ops_output_names.extend(full_ops_output_names_sub)
+    onnx_outputs_for_validation: List[np.ndarray] = dummy_onnx_inference(
+        onnx_graph=onnx_graph,
+        output_names=full_ops_output_names,
+    )
+    """
+    onnx_tensor_infos_for_validation:
+        {
+            onnx_output_name: np.ndarray,
+            onnx_output_name: np.ndarray,
+            onnx_output_name: np.ndarray,
+                        :
+        }
+    """
+    onnx_tensor_infos_for_validation = {
+        ops_output_name: onnx_output_for_validation \
+            for ops_output_name, onnx_output_for_validation \
+                in zip(full_ops_output_names, onnx_outputs_for_validation)
+    }
+
     if not non_verbose:
         print('')
         print(f'{Color.REVERCE}Model loaded{Color.RESET}', '=' * 72)
@@ -643,6 +672,7 @@ def convert(
         'replacement_parameters': replacement_parameters,
         'mvn_epsilon': mvn_epsilon,
         'output_signaturedefs': output_signaturedefs,
+        'onnx_tensor_infos_for_validation': onnx_tensor_infos_for_validation,
     }
 
     tf_layers_dict = {}
@@ -720,16 +750,13 @@ def convert(
             )
 
         # List "optype"="Input"
-        inputs = [
-            layer_info['op'] \
-                for layer_info in tf_layers_dict.values() \
-                    if layer_info['optype'] == 'Input'
-        ]
-        outputs = [
-            layer_info['tf_node'] \
-                for opname, layer_info in tf_layers_dict.items() \
-                    if opname in output_names
-        ]
+        inputs = get_tf_model_inputs(
+            tf_layers_dict=tf_layers_dict,
+        )
+        outputs = get_tf_model_outputs(
+            tf_layers_dict=tf_layers_dict,
+            output_names=output_names,
+        )
 
         model = tf.keras.Model(inputs=inputs, outputs=outputs)
         if not non_verbose:
@@ -1093,13 +1120,9 @@ def convert(
             # ]
 
             # Output OP extended to all ONNX nodes
+            ops_output_names = []
             if check_onnx_tf_outputs_elementwise_close_full:
-                ops_output_names = []
-                for graph_node in graph.nodes:
-                    ops_output_names_sub = []
-                    for graph_node_output in graph_node.outputs:
-                        ops_output_names_sub.append(graph_node_output.name)
-                    ops_output_names.extend(ops_output_names_sub)
+                ops_output_names = full_ops_output_names
             else:
                 ops_output_names = output_names
 
@@ -1158,26 +1181,16 @@ def convert(
                 test_data_nhwc=test_data_nhwc,
             )
             # TF dummy inference
-            dummy_tf_outputs: List[Any] = dummy_tf_inference(
+            tf_tensor_infos: Dict[Any] = dummy_tf_inference(
                 model=model,
                 inputs=inputs,
                 test_data_nhwc=test_data_nhwc,
             )
-            if not isinstance(dummy_tf_outputs, list):
-                dummy_tf_outputs = [dummy_tf_outputs]
-            dummy_tf_outputs = [
-                dummy_tf_output.numpy() \
-                    for dummy_tf_output in dummy_tf_outputs
-            ]
             # Validation
             onnx_tensor_infos = {
                 output_name: dummy_onnx_output \
                     for output_name, dummy_onnx_output in zip(ops_output_names, dummy_onnx_outputs)
             }
-            tf_tensor_infos = {
-                output_name: dummy_tf_output \
-                    for output_name, dummy_tf_output in zip(ops_output_names, dummy_tf_outputs)
-            }
             """
             np.allclose(
                 dummy_onnx_outputs,
@@ -1187,7 +1200,7 @@ def convert(
                 equal_nan=True,
             )
 
-            check_results: Dict[str, List[np.ndarray, bool]]
+            check_results: Dict[str, List[np.ndarray, int, float|int]]
                 {
                     onnx_output_name: [
                         onnx_tensor,
@@ -1196,13 +1209,18 @@ def convert(
                     ]
                 }
             """
+            input_names = [k.name for k in inputs]
+            onnx_tf_output_pairs = {
+                (k, v['tf_node'].name): (onnx_tensor_infos[k], tf_tensor_infos[v['tf_node'].name])
+                    for k, v in tf_layers_dict.items() if k not in input_names and not hasattr(v['tf_node'], 'numpy')
+            }
+
             check_results = onnx_tf_tensor_validation(
-                onnx_tensor_infos=onnx_tensor_infos,
-                tf_tensor_infos=tf_tensor_infos,
+                output_pairs=onnx_tf_output_pairs,
                 rtol=check_onnx_tf_outputs_elementwise_close_rtol,
                 atol=check_onnx_tf_outputs_elementwise_close_atol,
             )
-            for onnx_output_name, checked_value in check_results.items():
+            for (onnx_output_name, tf_output_name), checked_value in check_results.items():
                 validated_onnx_tensor: np.ndarray = checked_value[0]
                 matched_flg: int = checked_value[1]
                 max_abs_err: Any = checked_value[2]
@@ -1223,6 +1241,7 @@ def convert(
                 print(
                     f'{Color.GREEN}INFO:{Color.RESET} '+
                     f'{Color.GREEN}onnx_output_name{Color.RESET}: {onnx_output_name} '+
+                    f'{Color.GREEN}tf_output_name{Color.RESET}: {tf_output_name} '+
                     f'{Color.GREEN}shape{Color.RESET}: {validated_onnx_tensor.shape} '+
                     f'{Color.GREEN}dtype{Color.RESET}: {validated_onnx_tensor.dtype} '+
                     f'{message}'
